/*
    SpeedButton component for Java
    Copyright (c) 1998 Alessandro Scotti
*/
package org.ascotti.yawn;

import java.awt.*;
import java.awt.event.*;

/*
*/
public class BitBtn extends Component
{
    static final int LEFT = 1;
    static final int TOP = 2;
    static final int RIGHT = 3;
    static final int BOTTOM = 4;

    private int fMargin = -1;
    private String fLabel = "";
    private int fSpacing = 8;
    private int fStyle = LEFT;
    private int fTag;
    private Image fImage;
    private ActionListener fActionListener;
    protected boolean fJava3D = false;
    protected boolean fPressed = false;
    protected boolean fMouseDown = false;

    public BitBtn()
    {
        this( "", null );
    }

    public BitBtn( String aLabel, Image aImage )
    {
        fLabel = aLabel;
        fImage = aImage;
        setFont( new Font("Dialog", Font.PLAIN, 12) );
        setBackground( SystemColor.control );

        // Event processing
        addMouseListener( new MouseAdapter() {
            public void mousePressed( MouseEvent e ) {
                if( ((e.getModifiers() & e.BUTTON1_MASK) != 0) && isEnabled() ) {
                    fMouseDown = true;
                    setPressed( true );
                }
                else if( fMouseDown ) {
                    fMouseDown = false;
                    setPressed( false );
                }
            }
            public void mouseReleased( MouseEvent e ) {
                if( fMouseDown && (e.getModifiers() & e.BUTTON1_MASK) != 0 ) {
                    fMouseDown = false;
                    setPressed( false );
                    if( contains( e.getX(), e.getY() ) )
                        fireAction();
                }
            }
        });
        addMouseMotionListener( new MouseMotionAdapter() {
            public void mouseDragged( MouseEvent e ) {
                if( fMouseDown )
                    setPressed( contains( e.getX(), e.getY() ) );
            }
        });
    }

    protected void setPressed( boolean aPressed )
    {
        if( fPressed != aPressed ) {
            fPressed = aPressed;
            repaint();
        }
    }

    protected void fireAction()
    {
        if( fActionListener != null ) {
            ActionEvent event = new ActionEvent( this, ActionEvent.ACTION_PERFORMED, fLabel );
            fActionListener.actionPerformed( event );
        }
    }

    //
    protected void doRefresh()
    {
        invalidate();
        repaint();
    }

    private boolean isStyleVertical() {
        return( (fStyle == TOP) || (fStyle == BOTTOM) );
    }

    //
    public void addActionListener( ActionListener l )
    {
        fActionListener = AWTEventMulticaster.add( fActionListener, l );
    }

    //
    public void removeActionListener( ActionListener l )
    {
        fActionListener = AWTEventMulticaster.remove( fActionListener, l );
    }

    //
    public Dimension getPreferredSize() {
        return new Dimension(32,32);
    }

    //
    public Dimension getMinimumSize() {
        return new Dimension(8,8);
    }

    //
    public void update( Graphics g )
    {
        paint( g );
    }

    //
    public void paint( Graphics g )
    {
        int width = getSize().width, height = getSize().height;
        int i;
        int imgx = 0, imgy = 0, imgw = 0, imgh = 0, txtx = 0, txty = 0, txtw = 0, txth = 0;
        int margin = fMargin, spacing = fSpacing;
        int txtmaxascent = 0;

        // Get information about text and image
        if( getFont() != null ) {
            FontMetrics fm = getFontMetrics( getFont() );
            txtw = fm.stringWidth( getLabel() );
            txth = fm.getAscent() + fm.getDescent();
            txtmaxascent = fm.getMaxAscent();
        }
        if( fImage != null ) {
            imgw = fImage.getWidth( null );
            imgh = fImage.getHeight( null );
        }

        // Get image and text placement
        if( isStyleVertical() ) {
            imgx = (width - imgw + 1) / 2;
            txtx = (width - txtw + 1) / 2;
        }
        else {
            imgy = (height - imgh + 1) / 2;
            txty = (height - txth + 1) / 2;
        }

        // Adjust margin and spacing
        if( margin == -1 ) {
            margin = ( isStyleVertical() ? height - imgh - txth : width - imgw - txtw );
            if( spacing == -1 ) {
                margin /= 3;
                spacing = margin;
            }
            else {
                margin = (margin - spacing + 1) / 2;
            }
        }
        else {
            if( spacing == -1 ) {
                spacing = ( isStyleVertical() ? height - imgh - txth : width - imgw - txtw );
                spacing = (spacing - margin) / 2;
            }
        }

        // Spacing not meaningful if text or image is missing
        if( fImage == null || txtw == 0 )
            spacing = 0;

        // Fill background
        g.setColor( getBackground() );
        g.fillRect( 0, 0, width, height );

        // Adjust values according to style
        switch( fStyle ) {
        case LEFT:
            imgx = margin;
            txtx = imgx + imgw + spacing;
            break;
        case TOP:
            imgy = margin;
            txty = imgy + imgh + spacing;
            break;
        case RIGHT:
            imgx = width - margin - imgw;
            txtx = imgx - spacing - txtw;
            break;
        case BOTTOM:
            imgy = height - margin - imgh;
            txty = imgy - spacing - txth;
        }
        if( fPressed ) {
            imgx += 2;
            imgy += 2;
            txtx += 2;
            txty += 2;
        }

        // Draw image and text
        if( fImage != null )
            g.drawImage( fImage, imgx, imgy, imgw, imgh, this );
        if( txtw > 0 ) {
            txty += txtmaxascent;
            if( isEnabled() ) {
                g.setColor( getForeground() );
                g.drawString( getLabel(), txtx, txty );
            }
            else {
                g.setColor( SystemColor.controlHighlight );
                g.drawString( getLabel(), txtx+1, txty+1 );
                g.setColor( SystemColor.controlShadow );
                g.drawString( getLabel(), txtx, txty );
            }
        }

        // Draw frame
        int x1 = 0, y1 = 0, x2 = width-1, y2 = height-1;

        if( fJava3D ) {
            g.setColor( getBackground() );
            g.draw3DRect( x1, y1, x2, y2, !fPressed );
        }
        else {
            SystemColor [] colorRaised = { SystemColor.controlDkShadow, SystemColor.controlShadow, SystemColor.controlLtHighlight, SystemColor.controlHighlight };
            SystemColor [] colorSunken = { SystemColor.controlLtHighlight, SystemColor.controlHighlight, SystemColor.controlDkShadow, SystemColor.controlShadow };

            for( i=0; i<2; i++ ) {
                g.setColor( fPressed ? colorSunken[i] : colorRaised[i] );
                g.drawLine( x2, y1, x2, y2 );
                g.drawLine( x1, y2, x2, y2 );
                g.setColor( fPressed ? colorSunken[i+2] : colorRaised[i+2] );
                g.drawLine( x1, y1, x2, y1 );
                g.drawLine( x1, y1, x1, y2 );
                x1++;
                y1++;
                x2--;
                y2--;
            }
        }
    }

    /* Enabled */
    public void setEnabled( boolean aEnabled )
    {
        if( aEnabled != isEnabled() ) {
            super.setEnabled( aEnabled );
            repaint();
        }
    }

    /* Image */
    public void setImage( Image aImage )
    {
        fImage = aImage;
        doRefresh();
    }

    /* Label */
    public String getLabel() { return fLabel; }
    public void setLabel( String aLabel )
    {
        if( !fLabel.equals(aLabel) ) {
            fLabel = aLabel;
            doRefresh();
        }
    }

    /* Margin */
    public int getMargin() { return fMargin; }
    public void setMargin( int aMargin )
    {
        if( fMargin != aMargin ) {
            fMargin = aMargin;
            doRefresh();
        }
    }

    /* Spacing */
    public int getSpacing() { return fSpacing; }
    public void setSpacing( int aSpacing )
    {
        if( fSpacing != aSpacing ) {
            fSpacing = aSpacing;
            doRefresh();
        }
    }

    /* Style */
    public int getStyle() { return fStyle; }
    public void setStyle( int aStyle )
    {
        if( fStyle != aStyle && aStyle >= 1 && aStyle <= 4 ) {
            fStyle = aStyle;
            doRefresh();
        }
    }

    /* Tag */
    public int getTag() { return fTag; }
    public void setTag( int aTag ) { fTag = aTag; }
}

